﻿using Infragistics.Windows.DockManager;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace Infragistics.Samples
{
	public enum RelativePanePosition
	{
		TabItemBefore,
		TabItemAfter,
		Left,
		Right,
		Top,
		Bottom,
		RootInsideLeft,
		RootInsideTop,
		RootInsideBottom,
		RootInsideRight,
		RootOutsideLeft,
		RootOutsideTop,
		RootOutsideBottom,
		RootOutsideRight,
	}

	public static class PaneHelpers
	{
		public static ContentPane CreateRelativePane(this ContentPane relativePane, RelativePanePosition relativePosition)
		{
			var newPane = CreateRelativePaneImpl(relativePane, relativePosition);

			if (newPane != null && relativePane.IsPinned == false)
				newPane.SetCurrentValue(ContentPane.IsPinnedProperty, false);

			return newPane;
		}

		#region CreateRelativePaneImpl
		private static ContentPane CreateRelativePaneImpl(ContentPane relativePane, RelativePanePosition position)
		{
			DependencyObject parentPaneChild = relativePane;
			DependencyObject parentForPane = null;

			switch (relativePane.PaneLocation)
			{
				case PaneLocation.Unpinned:
					{
						// one cannot create a pane in the unpinned tab area so you have to find 
						// the placeholder that represents the pane in the docked location and 
						// add the new pane relative to that
						parentPaneChild = FindDockedPlaceholder(relativePane);

						if (parentPaneChild != null)
							parentForPane = LogicalTreeHelper.GetParent(parentPaneChild);

						break;
					}
				case PaneLocation.Unknown:
					Debug.Fail("The specified pane doesn't exist within a dockmanager.");
					return null;
				default:
					parentForPane = LogicalTreeHelper.GetParent(relativePane);
					break;
			}

			var parentSplitPane = parentForPane as SplitPane;
			var parentTabGroup = parentForPane as TabGroupPane;

			if (parentTabGroup != null)
			{
				switch (position)
				{
					case RelativePanePosition.TabItemAfter:
					case RelativePanePosition.TabItemBefore:
						var newPane = new ContentPane();
						var index = parentTabGroup.Items.IndexOf(parentPaneChild) + (IsPositionBefore(position) ? 0 : 1);
						parentTabGroup.Items.Insert(index, newPane);
						return newPane;

					default:
						parentSplitPane = LogicalTreeHelper.GetParent(parentTabGroup) as SplitPane;
						parentPaneChild = parentTabGroup;
						Debug.Assert(parentSplitPane != null, "The specified pane is within a TabGroupPane that isn't within a SplitPane.");
						break;
				}
			}


			if (parentSplitPane != null)
			{
				var newPane = new ContentPane();
				var parentPaneChildElement = (FrameworkElement)parentPaneChild;
				var targetIndex = parentSplitPane.Panes.IndexOf(parentPaneChildElement);

				switch (position)
				{
					case RelativePanePosition.RootInsideLeft:
					case RelativePanePosition.RootInsideRight:
					case RelativePanePosition.RootInsideTop:
					case RelativePanePosition.RootInsideBottom:
					case RelativePanePosition.RootOutsideLeft:
					case RelativePanePosition.RootOutsideRight:
					case RelativePanePosition.RootOutsideTop:
					case RelativePanePosition.RootOutsideBottom:
						{
							var rootSplit = parentSplitPane;

							while (true)
							{
								var parentSplit = LogicalTreeHelper.GetParent(rootSplit) as SplitPane;

								if (parentSplit == null)
									break;

								rootSplit = parentSplit;
							}

							var dm = XamDockManager.GetDockManager(relativePane);
							var rootIndex = dm.Panes.IndexOf(rootSplit);

							Debug.Assert(rootIndex >= 0 || relativePane.PaneLocation == PaneLocation.Document, "Cannot find the root split pane to use as the relative position");

							if (rootIndex < 0) // if it was in the document area...
								rootIndex = dm.Panes.Count;
							else if (!IsPositionBefore(position))
								rootIndex++;

							var newSplit = new SplitPane();
							XamDockManager.SetInitialLocation(newSplit, GetRootLocation(position).Value);
							newSplit.Panes.Add(newPane);
							dm.Panes.Insert(rootIndex, newSplit);
							return newPane;
						}
					case RelativePanePosition.TabItemBefore:
					case RelativePanePosition.TabItemAfter:
						{
							// remove what will be the new sibling from the old location
							parentSplitPane.Panes.Remove(parentPaneChildElement);

							// the parent is a splitpane but we need a tab group so create one
							parentTabGroup = new TabGroupPane();
							parentTabGroup.Items.Add(parentPaneChild);
							parentTabGroup.Items.Insert(IsPositionBefore(position) ? 0 : 1, newPane);

							parentSplitPane.Panes.Insert(targetIndex, parentTabGroup);

							return newPane;
						}
					default:
						{
							var requiredSplitOrientation = GetSplitterOrientation(position);
							var newChild = (FrameworkElement)newPane;

							// if the target is a document then we have to wrap it in a tabgroup since all panes in a 
							// document content host must be within a tabgrouppane
							if (relativePane.PaneLocation == PaneLocation.Document)
							{
								var newTabGroup = new TabGroupPane();
								newTabGroup.Items.Add(newPane);
								newChild = newTabGroup;
							}

							// if the pane only has the 1 child and its orientation isn't what's required
							// then we can just alter the property itself
							if (parentSplitPane.Panes.Count == 1 && requiredSplitOrientation != parentSplitPane.SplitterOrientation)
								parentSplitPane.SplitterOrientation = requiredSplitOrientation.Value;

							// that split is in the required orientation just add it as a sibling
							if (requiredSplitOrientation == parentSplitPane.SplitterOrientation)
							{
								if (position == RelativePanePosition.Right || position == RelativePanePosition.Bottom)
									targetIndex++;

								parentSplitPane.Panes.Insert(targetIndex, newChild);
							}
							else
							{
								parentSplitPane.Panes.Remove(parentPaneChildElement);

								// otherwise need a new split
								var newSplit = new SplitPane();
								newSplit.SplitterOrientation = requiredSplitOrientation.Value;
								newSplit.Panes.Add(parentPaneChildElement);
								newSplit.Panes.Insert(IsPositionBefore(position) ? 0 : 1, newChild);

								parentSplitPane.Panes.Insert(targetIndex, newSplit);
							}

							return newPane;
						}
				}
			}

			return null;
		} 
		#endregion

		#region GetRootLocation
		private static InitialPaneLocation? GetRootLocation(RelativePanePosition position)
		{
			switch (position)
			{
				case RelativePanePosition.RootInsideLeft:
				case RelativePanePosition.RootOutsideLeft:
					return InitialPaneLocation.DockedLeft;

				case RelativePanePosition.RootInsideRight:
				case RelativePanePosition.RootOutsideRight:
					return InitialPaneLocation.DockedRight;

				case RelativePanePosition.RootInsideTop:
				case RelativePanePosition.RootOutsideTop:
					return InitialPaneLocation.DockedTop;

				case RelativePanePosition.RootInsideBottom:
				case RelativePanePosition.RootOutsideBottom:
					return InitialPaneLocation.DockedBottom;

				default:
					return null;
			}
		} 
		#endregion

		#region IsPositionBefore
		private static bool IsPositionBefore(RelativePanePosition position)
		{
			switch (position)
			{
				case RelativePanePosition.RootOutsideBottom:
				case RelativePanePosition.RootOutsideTop:
				case RelativePanePosition.RootOutsideLeft:
				case RelativePanePosition.RootOutsideRight:
				case RelativePanePosition.TabItemBefore:
				case RelativePanePosition.Left:
				case RelativePanePosition.Top:
					return true;
				default:
					return false;
			}
		} 
		#endregion

		#region GetSplitterOrientation
		private static Orientation? GetSplitterOrientation(RelativePanePosition position)
		{
			switch (position)
			{
				default:
					return null;
				case RelativePanePosition.Left:
				case RelativePanePosition.Right:
					return Orientation.Vertical;
				case RelativePanePosition.Top:
				case RelativePanePosition.Bottom:
					return Orientation.Horizontal;
			}
		} 
		#endregion

		#region FindDockedPlaceholder
		private static ContentPanePlaceholder FindDockedPlaceholder(ContentPane pane)
		{
			var dm = XamDockManager.GetDockManager(pane);

			foreach (var rootPane in dm.Panes)
			{
				switch (XamDockManager.GetPaneLocation(rootPane))
				{
					case PaneLocation.DockedBottom:
					case PaneLocation.DockedLeft:
					case PaneLocation.DockedRight:
					case PaneLocation.DockedTop:
						var placeholder = FindPlaceholder(rootPane, pane);

						if (placeholder != null)
							return placeholder;

						break;
					default:
						// only looking for docked panes
						continue;
				}
			}

			return null;
		} 
		#endregion

		#region FindPlaceholder
		private static ContentPanePlaceholder FindPlaceholder(SplitPane pane, ContentPane contentPane)
		{
			foreach (var child in pane.Panes)
			{
				ContentPanePlaceholder placeholder = null;

				if (child is TabGroupPane)
					placeholder = FindPlaceholder(child as TabGroupPane, contentPane);
				else if (child is SplitPane)
					placeholder = FindPlaceholder(child as SplitPane, contentPane);
				else
					placeholder = child as ContentPanePlaceholder;

				if (placeholder != null && placeholder.Pane == contentPane)
					return placeholder;
			}

			return null;
		}

		private static ContentPanePlaceholder FindPlaceholder(TabGroupPane pane, ContentPane contentPane)
		{
			foreach (var item in pane.Items)
			{
				var placeholder = item as ContentPanePlaceholder;

				if (placeholder != null && placeholder.Pane == contentPane)
					return placeholder;
			}

			return null;
		} 
		#endregion //FindPlaceholder
	}
}
